=== modified file 'lib/lp/archivepublisher/domination.py'
--- lib/lp/archivepublisher/domination.py	2011-11-04 09:29:43 +0000
+++ lib/lp/archivepublisher/domination.py	2011-11-04 09:47:37 +0000
@@ -54,6 +54,10 @@
 
 from collections import defaultdict
 from datetime import timedelta
+from itertools import (
+    ifilter,
+    ifilterfalse,
+    )
 from operator import (
     attrgetter,
     itemgetter,
@@ -201,15 +205,15 @@
         """Sort publications from most to least current versions."""
         # Listify; we want to iterate this twice, which won't do for a
         # non-persistent sequence.
-        sorted_publications = list(publications)
+        publications = list(publications)
         # Batch-load associated package releases; we'll be needing them
         # to compare versions.
-        self.load_releases(sorted_publications)
+        self.load_releases(publications)
         # Now sort.  This is that second iteration.  An in-place sort
         # won't hurt the original, because we're working on a copy of
         # the original iterable.
-        sorted_publications.sort(cmp=self.compare, reverse=True)
-        return sorted_publications
+        publications.sort(cmp=self.compare, reverse=True)
+        return publications
 
 
 def find_live_source_versions(publications):
@@ -228,7 +232,12 @@
 
 
 def get_binary_versions(binary_publications):
-    """List versions for sequence of `BinaryPackagePublishingHistory`."""
+    """List versions for sequence of `BinaryPackagePublishingHistory`.
+
+    :param binary_publications: An iterable of
+        `BinaryPackagePublishingHistory`.
+    :return: A list of the publications' respective versions.
+    """
     return [pub.binarypackagerelease.version for pub in binary_publications]
 
 
@@ -280,10 +289,8 @@
     publications = list(publications)
     latest = publications.pop(0)
     is_arch_specific = attrgetter('architecture_specific')
-    arch_specific_pubs = filter(is_arch_specific, publications)
-    arch_indep_pubs = filter(
-        lambda pub: not is_arch_specific(pub),
-        publications)
+    arch_specific_pubs = list(ifilter(is_arch_specific, publications))
+    arch_indep_pubs = list(ifilterfalse(is_arch_specific, publications))
 
     # XXX JeroenVermeulen 2011-11-01 bug=884649: This is likely to be
     # costly, and the result could be reused for all builds of the same
@@ -641,8 +648,12 @@
         """Find binary publications that need dominating.
 
         This is only for traditional domination, where the latest published
-        publication is always kept published.  It will ignore publications
-        that have no other publications competing for the same binary package.
+        publication is always kept published.  See `find_live_source_versions`
+        for this logic.
+
+        To optimize for that logic, `findSourcesForDomination` will ignore
+        publications that have no other publications competing for the same
+        binary package.  There'd be nothing to do for those cases.
         """
         # Avoid circular imports.
         from lp.soyuz.model.publishing import SourcePackagePublishingHistory
@@ -690,7 +701,7 @@
         self.logger.debug("Dominating sources...")
         for name, pubs in sorted_packages.iteritems():
             self.logger.debug("Dominating %s" % name)
-            assert len(pubs) > 0, "Dominating zero binaries!"
+            assert len(pubs) > 0, "Dominating zero sources!"
             live_versions = find_live_source_versions(pubs)
             self.dominatePackage(pubs, live_versions, generalization)
 

=== modified file 'lib/lp/archivepublisher/tests/test_dominator.py'
--- lib/lp/archivepublisher/tests/test_dominator.py	2011-11-03 11:25:13 +0000
+++ lib/lp/archivepublisher/tests/test_dominator.py	2011-11-04 09:47:37 +0000
@@ -1146,15 +1146,23 @@
     layer = ZopelessDatabaseLayer
 
     def test_find_live_source_versions_blesses_latest(self):
+        # find_live_source_versions, assuming that you passed it
+        # publications sorted from most current to least current
+        # version, simply returns the most current version.
         spphs = make_spphs_for_versions(self.factory, ['1.2', '1.1', '1.0'])
         self.assertEqual(['1.2'], find_live_source_versions(spphs))
 
     def test_find_live_binary_versions_pass_1_blesses_latest(self):
+        # find_live_binary_versions_pass_1 always includes the latest
+        # version among the input publications in its result.
         bpphs = make_bpphs_for_versions(self.factory, ['1.2', '1.1', '1.0'])
         make_publications_arch_specific(bpphs)
         self.assertEqual(['1.2'], find_live_binary_versions_pass_1(bpphs))
 
     def test_find_live_binary_versions_pass_1_blesses_arch_all(self):
+        # find_live_binary_versions_pass_1 includes any
+        # architecture-independent publications among the input in its
+        # result.
         versions = list(reversed(['1.%d' % version for version in range(3)]))
         bpphs = make_bpphs_for_versions(self.factory, versions)
 
@@ -1169,11 +1177,16 @@
             find_live_binary_versions_pass_1(bpphs))
 
     def test_find_live_binary_versions_pass_2_blesses_latest(self):
+        # find_live_binary_versions_pass_2 always includes the latest
+        # version among the input publications in its result.
         bpphs = make_bpphs_for_versions(self.factory, ['1.2', '1.1', '1.0'])
         make_publications_arch_specific(bpphs, False)
         self.assertEqual(['1.2'], find_live_binary_versions_pass_2(bpphs))
 
     def test_find_live_binary_versions_pass_2_blesses_arch_specific(self):
+        # find_live_binary_versions_pass_2 includes any
+        # architecture-specific publications among the input in its
+        # result.
         versions = list(reversed(['1.%d' % version for version in range(3)]))
         bpphs = make_bpphs_for_versions(self.factory, versions)
         make_publications_arch_specific(bpphs)

